package com.starbuds.server.api.demo.extension;

import com.starbuds.server.api.demo.constant.HttpError;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.validation.ConstraintViolationException;
import javax.ws.rs.NotAllowedException;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;

import static com.starbuds.server.api.demo.util.SpringContextUtil.isDev;

/**
 * 自定义异常捕获
 *
 * @author Lay
 * @date 2019-02-18
 */
@Slf4j
@Component
@Provider
@Produces(ContentType.APPLICATION_JSON_UTF_8)
public class CustomExceptionMapper implements ExceptionMapper<Exception> {

    @Override
    public Response toResponse(Exception e) {
        log.error("\nexception===", e);

        if (e instanceof NotFoundException) {
            log.error("无效的接口路径");
            return HttpError.NOT_FOUND;

        } else if (e instanceof NumberFormatException || e instanceof javax.ws.rs.BadRequestException) {
            log.error("无效的参数");
            return HttpError.BAD_REQUEST;

        } else if (e instanceof NotAllowedException) {
            log.error("无效的请求方式");
            return HttpError.METHOD_NOT_ALLOWED;

        } else if (e instanceof ConstraintViolationException) {
            String errMsg = ((ConstraintViolationException) e).getConstraintViolations().stream()
                    .findAny()
                    .map(set -> set.getMessage())
                    .orElse("参数错误");
            return Response.status(Response.Status.OK).entity(Result.ofBadRequest(errMsg)).type(ContentType.APPLICATION_JSON_UTF_8).build();

        }else {
            if(isDev()){
                StackTraceElement stackTraceElement = e.getStackTrace()[0];
                System.out.println("exception File==="+stackTraceElement.getFileName());
                System.out.println("exception Line==="+stackTraceElement.getLineNumber());

                String msg = "服务异常，请稍后再试 "+ e.getStackTrace().getClass() + e.toString();
                return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(new Result(GlobalApiError.ServerError,msg)).type(ContentType.APPLICATION_JSON_UTF_8).build();
            }else{
                return HttpError.SERVER_ERROR;
            }
        }
    }
}