package com.starbuds.server.api.demo.extension;

import com.starbuds.server.api.demo.constant.HttpError;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Priority;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.core.Context;
import javax.ws.rs.ext.Provider;
import java.io.IOException;

/**
 * 官方管理员操作拦截器
 *
 * @author aoe
 * @date 2022/1/7
 */
@Slf4j
@Component
@Provider
@Priority(2)
public class OfficialInterceptor implements ContainerRequestFilter {

    @Context
    private HttpServletRequest request;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Override
    public void filter(ContainerRequestContext requestContext) throws IOException {
        String reqPath = requestContext.getUriInfo().getPath();
        if (!reqPath.startsWith("/v1/official")) {
            return;
        }

        Long userId = userId();
        if (userId == null) {
            log.warn("官方用户id是null");
            requestContext.abortWith(HttpError.ACCESS_DENIED);
        }

        boolean isOfficial = userService.isOfficial(userId);
        if (!isOfficial) {
            log.warn("无效的官方用户id = {}", userId);
            requestContext.abortWith(HttpError.ACCESS_DENIED);
        }
    }

    private Long userId() {
        TokenBO token = getToken();
        if (token == null) {
            return null;
        }
        return token.getUserId();
    }

    private TokenBO getToken() {
        Object token = request.getAttribute("token");
        if (token != null) {
            return (TokenBO) token;
        }
        return null;
    }

}
