package com.starbuds.server.api.demo.util;

import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Locale;

import static com.starbuds.server.common.constant.i18n.I18nTypeEnum.ZH_CN;

/**
 * 国际化工具
 *
 * @author aoe
 * @date 2020/5/12
 */
@Component
public class I18nUtil {

    @Resource
    private MessageSource messageSource;

    /**
     * key 特殊字符转义（这些字符不能包含在key中）
     */
    private String[] escapeArr = new String[]{":", "："};

    /**
     * 国际化消息
     *
     * @param key
     * @param lang
     * @return
     */
    public String getMsg(String key, String lang) {
        I18nTypeEnum i18nTypeEnum = I18nTypeEnum.toEnum(lang);
        if (ZH_CN == i18nTypeEnum) {
            return key;
        }
        Locale locale = Locale.CHINA;
        switch (i18nTypeEnum) {
            case EN:
                locale = Locale.ENGLISH;
                break;
        }
        String message = messageSource.getMessage(getKey(key), null, key, locale);
        return message;
    }

    public String getKey(String key) {
        if (StringUtils.isBlank(key)) {
            return "KeyIsBlank";
        }
        // key中不能有空格
        key = key.replaceAll(" ", "");
        for (String escape : escapeArr) {
            key = key.replaceAll(escape, "1XEscapeX1");
        }

        return key;
    }
}
